package org.zen.xhtml.editor;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentPartitioner;
import org.eclipse.jface.text.IDocumentPartitioningListener;
import org.eclipse.jface.text.IDocumentPartitioningListenerExtension;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.rules.DefaultPartitioner;
import org.eclipse.ui.editors.text.FileDocumentProvider;
import org.zen.xhtml.editor.parser.SAXHTMLParser;
import org.zen.xhtml.editor.scanner.XHTMLPartitionScanner;

/**
 * @author Zenil
 * @version 1.0
 */
public class XHTMLDocumentProvider extends FileDocumentProvider {

	private IDocumentPartitioner fPartitioner;
	
	/**
	 * Constructor for XHTMLDocumentProvider.
	 */
	public XHTMLDocumentProvider() {
		super();
	}

	/* (non-Javadoc)
	 * Method declared on AbstractDocumentProvider
	 */
	protected IDocument createDocument(Object element) throws CoreException {
		IDocument document = super.createDocument(element);
		if (document != null) {
			fPartitioner =
				new DefaultPartitioner(new XHTMLPartitionScanner(), new String[] {IDocument.DEFAULT_CONTENT_TYPE, IXHTMLEditorConstants.CONTENT_TYPE_HTML_TAG, IXHTMLEditorConstants.CONTENT_TYPE_HTML_COMMENT,IXHTMLEditorConstants.CONTENT_TYPE_HTML_PI});

			fPartitioner.connect(document);
			
			//add the partition listener before setting the partitioner!
			//only then will notfication happens.
			//The listener is useful for testing whether proper partitioning
			//is happening or not
			document.addDocumentPartitioningListener(new MyPartitionListener());
			document.setDocumentPartitioner(fPartitioner);					
		}
		
		return document;
	}	
	
}
class MyPartitionListener implements IDocumentPartitioningListener, IDocumentPartitioningListenerExtension {

	/**
	 * @see org.eclipse.jface.text.IDocumentPartitioningListener#documentPartitioningChanged(IDocument)
	 */
	public void documentPartitioningChanged(IDocument document) {
	}

	/**
	 * @see org.eclipse.jface.text.IDocumentPartitioningListenerExtension#documentPartitioningChanged(IDocument, IRegion)
	 */
	public void documentPartitioningChanged(IDocument document, IRegion region) {
//		System.out.println("Length = " + region.getLength() + " Offset = " + region.getOffset());
//		try {
//			System.out.println(
//				"Content Type = " + document.getContentType(region.getOffset()) + " Line = " + document.getLineOfOffset(region.getOffset()));
//			System.out.println("String = " + document.get(region.getOffset(), region.getLength()));
//		} catch (BadLocationException e) {
//		}
//		System.out.println("--------------------------------------------------------------------------");
	}

}
