package org.zen.xhtml.editor;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.editors.text.TextEditorActionContributor;
import org.eclipse.ui.internal.IWorkbenchGraphicConstants;
import org.eclipse.ui.internal.WorkbenchImages;
import org.eclipse.ui.texteditor.IAbstractTextEditorHelpContextIds;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.texteditor.ITextEditorActionConstants;
import org.eclipse.ui.texteditor.RetargetTextEditorAction;

/**
 * Without an BasicTextEditorActionContributor ,the common actions like
 * cut,copy,paste,gotoline,find,find Replace in the menubar will not be enabled.
 * You can specify the actioncontributor either via plugin.xml or
 * via <<editorClass>>.setActionContributor().
 *
 * @author Zenil
 * @version 1.0
 * @see EditorActionBarContributor
 */
public class XHTMLEditorContribution extends TextEditorActionContributor {

	/**
	 * Place holder for Cut,Copy,Paste Actions
	 * These actions have already been created in AbstractTextEditor
	 * and contributed to Menubar by BasicTextEditorActionContributor. 
	 * Contributing these actions in the toolbar also.
	 */
	private RetargetTextEditorAction fCutAction,fCopyAction,fPasteAction;
	
	/**
	 * Place holder for HTML information display action (by pressing F2)
	 * This action has already been created in XHTMLEditor.The XHTMLEditorContribution
	 * will contribute this action to the MenuBar
	 */
	private RetargetTextEditorAction fShowHTMLInfo;
	
	private RetargetTextEditorAction fContentAssistProposal;
	private RetargetTextEditorAction fContentAssistTip;
	
	public XHTMLEditorContribution() {
		super();
		fCutAction = new RetargetTextEditorAction(XHTMLEditorPlugin.getDefault().getResourceBundle(),"Cut.");//$NON-NLS-1$ This is just a dummy, as actual label,tooltip is already defiend by cut action in AbstractTextEditor
		fCutAction.setImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_CUT_EDIT));
		fCutAction.setHoverImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_CUT_EDIT_HOVER));
		fCutAction.setDisabledImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_CUT_EDIT_DISABLED));
		//this is useless as help context id's do not seem to work for toolbars
		fCutAction.setHelpContextId(IAbstractTextEditorHelpContextIds.CUT_ACTION);

		fCopyAction = new RetargetTextEditorAction(XHTMLEditorPlugin.getDefault().getResourceBundle(),"Copy.");//$NON-NLS-1$ This is just a dummy, as actual label,tooltip is already defiend by cut action in AbstractTextEditor
		fCopyAction.setImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_COPY_EDIT));
		fCopyAction.setHoverImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_COPY_EDIT_HOVER));
		fCopyAction.setDisabledImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_COPY_EDIT_DISABLED));
		
		fPasteAction = new RetargetTextEditorAction(XHTMLEditorPlugin.getDefault().getResourceBundle(),"Paste.");//$NON-NLS-1$ This is just a dummy, as actual label,tooltip is already defiend by cut action in AbstractTextEditor
		fPasteAction.setImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_PASTE_EDIT));
		fPasteAction.setHoverImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_PASTE_EDIT_HOVER));
		fPasteAction.setDisabledImageDescriptor(WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_CTOOL_PASTE_EDIT_DISABLED));
		
		fShowHTMLInfo= new RetargetTextEditorAction(XHTMLEditorPlugin.getDefault().getResourceBundle(), "ShowHTMLInfo."); //$NON-NLS-1$
		
		fContentAssistProposal= new RetargetTextEditorAction(XHTMLEditorPlugin.getDefault().getResourceBundle(),"ContentAssistProposal."); //$NON-NLS-1$
		fContentAssistTip= new RetargetTextEditorAction(XHTMLEditorPlugin.getDefault().getResourceBundle(), "ContentAssistTip."); //$NON-NLS-1$
		
		//check alternateMethod() for getting fCutAction() below
	}
	
	
	/**
	 * Super class method adds the global action handler
	 * Setting the fCutAction to the proper action
	 * @see org.eclipse.ui.IEditorActionBarContributor#setActiveEditor(IEditorPart)
	 */
	public void setActiveEditor(IEditorPart part) {
		
		//if the same editor is getting activated ,then return!
		IEditorPart activeEditorPart = getActiveEditorPart();
		if (activeEditorPart == part)
			return;
			
		super.setActiveEditor(part);
		
		ITextEditor editor= (part instanceof ITextEditor) ? (ITextEditor) part : null;
		
		//set the proper underlying actions here!
		fCutAction.setAction(getAction(editor,ITextEditorActionConstants.CUT))	;			
		fCopyAction.setAction(getAction(editor,ITextEditorActionConstants.COPY))	;
		fPasteAction.setAction(getAction(editor,ITextEditorActionConstants.PASTE))	;
		
		fShowHTMLInfo.setAction(getAction(editor,IXHTMLEditorConstants.ACTION_HTML_INFO));
		
		fContentAssistProposal.setAction(getAction(editor, IXHTMLEditorConstants.ACTION_CONTENT_ASSIST_PROPOSAL)); //$NON-NLS-1$
		fContentAssistTip.setAction(getAction(editor, IXHTMLEditorConstants.ACTION_CONTENT_ASSIST_TIP)); //$NON-NLS-1$

	}
	
	/*
	 * @see EditorActionBarContributor#contributeToMenu(IMenuManager)
	 */
	public void contributeToMenu(IMenuManager menu) {
		
		super.contributeToMenu(menu);
		
		IMenuManager editMenu= menu.findMenuUsingPath(IWorkbenchActionConstants.M_EDIT);
		if (editMenu != null) {
			editMenu.add(new Separator(IXHTMLEditorConstants.GROUP_ASSIST));
			editMenu.appendToGroup(IXHTMLEditorConstants.GROUP_ASSIST,fContentAssistProposal);
			editMenu.appendToGroup(IXHTMLEditorConstants.GROUP_ASSIST,fContentAssistTip);
			editMenu.appendToGroup(IXHTMLEditorConstants.GROUP_ASSIST, fShowHTMLInfo);
		}		
	}
	
	/**
	 * @see org.eclipse.ui.part.EditorActionBarContributor#contributeToToolBar(IToolBarManager)
	 */
	public void contributeToToolBar(IToolBarManager toolBarManager) {

		// Run super.
		super.contributeToToolBar(toolBarManager);
	
		// Add Cut,Copy,Paste to toolbar
		toolBarManager.add(new Separator(IXHTMLEditorConstants.GROUP_ASSIST));
		toolBarManager.add(fCutAction);		
		toolBarManager.add(fCopyAction);
		toolBarManager.add(fPasteAction);
	}
}

	/*
	 * alternateMethod() {
	 * 
	 * IWorkbenchWindow window = XHTMLEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow();
	 * WorkbenchWindow win = (WorkbenchWindow)window;
	 * HashMap map = win.getKeyBindingService().getMapping();
	 * RetargetAction fCutAction = (RetargetAction)map.get(ITextEditorActionDefinitionIds.CUT);
	 * }
	 * Expl:
	 * Cut,Copy,Paste.. actions are global actions.They are created very first
	 * in WorkbenchActionBuilder before any editors or views are created.
	 * Therefore we can get a handle to these actions in the constructor itself and 
	 * directlty use them in the  contributeToToolBar().No need of overriding setActiveEditor
	 * 
	 * The downside is we are using internal 
	 * eclipse classes that is not recommended.Therefore we have to go in a roundabout
	 * way  
	 */

