package org.zen.xhtml.editor;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.internal.WorkbenchPlugin;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.zen.xhtml.editor.preferencePage.IPreferenceConstants;

/**
 * 
 * @author Zenil
 * @version 1.0
 *
 * The main plugin class to be used in the desktop.
 */
public class XHTMLEditorPlugin extends AbstractUIPlugin {
	//The shared instance.
	private static XHTMLEditorPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	
	/**
	 * The constructor.
	 */
	public XHTMLEditorPlugin(IPluginDescriptor descriptor) {
		super(descriptor);
		plugin = this;
		try {
			resourceBundle = ResourceBundle.getBundle("org.zen.xhtml.editor.EditorPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/**
	 * Returns the shared instance.
	 */
	public static XHTMLEditorPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = XHTMLEditorPlugin.getDefault().getResourceBundle();
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	/**
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#initializeDefaultPluginPreferences()
	 */
	protected void initializeDefaultPluginPreferences() {
		/*
		 * Using the workbench's preference store ,to gain workbench font
		 * functionality as separate font functionality not implemented
		 * Ideally should use the plugins preference store.
		 * Preferences pref = getPluginPreferences().
		 */
		Preferences pref = WorkbenchPlugin.getDefault().getPluginPreferences();
		pref.setDefault(IPreferenceConstants.VRULER_PREFERENCE, true);
		pref.setDefault(IPreferenceConstants.HRULER_PREFERENCE, true);
	}

	/**
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#getPreferenceStore()
	 */
	/*
	 * Using the workbench's preference store ,to gain workbench font
	 * functionality as separate font functionality not implemented
	 * Ideally should use the plugins preference store.
	 * Preferences pref = getPluginPreferences().
	 */
	public IPreferenceStore getPreferenceStore() {
		return WorkbenchPlugin.getDefault().getPreferenceStore();
	}	
	
	/**
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#shutdown()
	 */
	public void shutdown() throws CoreException {
		super.shutdown();
		ColorManager.getColorManager().dispose();
	}

}
