package org.zen.xhtml.editor;

import javax.swing.text.html.HTML;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextHover;
import org.eclipse.jface.text.ITextViewer;
import org.zen.xhtml.editor.util.XHTMLPrinter;
import org.zen.xhtml.editor.util.XHTMLWordFinder;

/**
 * 
 * @author Zenil
 * @version 1.0
 *
 * Example implementation for an <code>ITextHover</code> which hovers over XHTML code.
 */
public class XHTMLTextHover implements ITextHover {

	/**
	 * @see org.eclipse.jface.text.ITextHover#getHoverInfo(ITextViewer, IRegion)
	 */
	public String getHoverInfo(ITextViewer textViewer, IRegion hoverRegion) {
		StringBuffer buffer = new StringBuffer();
		if (hoverRegion != null) {
			try {
				if (hoverRegion.getLength() > -1) {
					String text = textViewer.getDocument().get(hoverRegion.getOffset(), hoverRegion.getLength());
					if (text != null && text.length() > 0) {
						XHTMLPrinter.addPageProlog(buffer);
		    			XHTMLPrinter.addSmallHeader(buffer,text);
				     	XHTMLPrinter.addPageEpilog(buffer);
					}

				}
			} catch (BadLocationException x) {
			}
		}
		return buffer.toString();
	}

	/**
	 * @see org.eclipse.jface.text.ITextHover#getHoverRegion(ITextViewer, int)
	 */
	public IRegion getHoverRegion(ITextViewer textViewer, int offset) {
		return XHTMLWordFinder.findWord(textViewer.getDocument(), offset);
	}	
}
