package org.zen.xhtml.editor.contentassist;

import java.text.MessageFormat;

import javax.swing.text.html.HTML;

import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.TextPresentation;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ContextInformation;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.jface.text.contentassist.IContextInformationPresenter;
import org.eclipse.jface.text.contentassist.IContextInformationValidator;
import org.zen.xhtml.editor.XHTMLEditorPlugin;

/**
 * @author Zenil
 * @version 1.0
 */
public class XHTMLContentAssistProcessor implements IContentAssistProcessor {

	private IContextInformationValidator fValidator= new Validator();
		
	/**
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#computeCompletionProposals(ITextViewer, int)
	 */
	public ICompletionProposal[] computeCompletionProposals(ITextViewer viewer, int documentOffset) {
		HTML.Tag tags[] = HTML.getAllTags();
		HTML.Attribute attrs[] = HTML.getAllAttributeKeys();
		ICompletionProposal[] result= new ICompletionProposal[tags.length+attrs.length];
		CompletionProposal cp =null;
		for (int i= 0; i < tags.length; i++) {
			String tag = tags[i].toString();
			IContextInformation info= new ContextInformation(tag, MessageFormat.format(XHTMLEditorPlugin.getResourceString("CompletionProcessor.Proposal.ContextInfo.pattern"), new Object[] { tag })); //$NON-NLS-1$
			cp = new CompletionProposal(tag, documentOffset, 0, tag.length(), null, tag, info,"Additional Proposal Information :".concat(MessageFormat.format(XHTMLEditorPlugin.getResourceString("CompletionProcessor.Proposal.hoverinfo.pattern"), new Object[] { tag})));
			result[i]= new XHTMLCompletionProposal(cp,documentOffset); //$NON-NLS-1$
		}
		for (int i= tags.length; i < (tags.length+attrs.length); i++) {
			String attr = attrs[i-tags.length].toString();
			IContextInformation info= new ContextInformation(attr, MessageFormat.format(XHTMLEditorPlugin.getResourceString("CompletionProcessor.Proposal.ContextInfo.pattern"), new Object[] { attr})); //$NON-NLS-1$
			cp = new CompletionProposal(attr, documentOffset, 0, attr.length(), null, attr, info, "Additional Proposal Information :".concat(MessageFormat.format(XHTMLEditorPlugin.getResourceString("CompletionProcessor.Proposal.hoverinfo.pattern"), new Object[] { attr})));
			result[i]= new XHTMLCompletionProposal(cp,documentOffset); //$NON-NLS-1$
		}
		return result;
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#computeContextInformation(ITextViewer, int)
	 */
	public IContextInformation[] computeContextInformation(ITextViewer viewer, int documentOffset) {
		ICompletionProposal[] proposals = computeCompletionProposals(viewer,documentOffset);
		IContextInformation[] result= new IContextInformation[proposals.length];
		for (int i= 0; i < result.length; i++)
			result[i]= proposals[i].getContextInformation();
		
		return result;
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#getCompletionProposalAutoActivationCharacters()
	 */
	public char[] getCompletionProposalAutoActivationCharacters() {
		return new char[] { '<'};
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#getContextInformationAutoActivationCharacters()
	 */
	public char[] getContextInformationAutoActivationCharacters() {
		return null;
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#getErrorMessage()
	 */
	public String getErrorMessage() {
		return "XHTMLContentAssistProcessor Error";//$NON-NLS-1$
	}

	/**
	 * After context information popup is shown,the following validator
	 * is used to hide the popup. The validator specifies a condition.
	 * The popup is shown only as long as that condition is true.
	 * When the condition becomes false,the pop up is hidden
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#getContextInformationValidator()
	 */
	public IContextInformationValidator getContextInformationValidator() {
		return fValidator;
	}

	/**
	 * Simple context information pop up closer. The popup window is valid in a range
	 * of 5 characters around its popup location.
	 */
	protected static class Validator implements IContextInformationValidator, IContextInformationPresenter {

		protected int fInstallOffset;

		/*
		 * @see IContextInformationValidator#isContextInformationValid(int)
		 */
		public boolean isContextInformationValid(int offset) {
			return Math.abs(fInstallOffset - offset) < 3;
		}

		/*
		 * @see IContextInformationValidator#install(IContextInformation, ITextViewer, int)
		 */
		public void install(IContextInformation info, ITextViewer viewer, int offset) {
			fInstallOffset= offset;
		}
		
		/*
		 * @see org.eclipse.jface.text.contentassist.IContextInformationPresenter#updatePresentation(int, TextPresentation)
		 */
		public boolean updatePresentation(int documentPosition, TextPresentation presentation) {
			return false;
		}
	};
}
