package org.zen.xhtml.editor.parser;

import java.util.ArrayList;
import java.util.HashMap;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Zenil
 * @version 1.0
 */
public class XHTMLTreeContentProvider implements ITreeContentProvider {

	private IDocumentProvider fProvider;
	private Document fDoc;
	private ArrayList fCountTags;

	public XHTMLTreeContentProvider(IDocumentProvider provider) {
		fProvider = provider;
		fCountTags = new ArrayList();
	}

	/**
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(Object)
	 */
	public Object[] getChildren(Object parentElement) {
		ArrayList alist = new ArrayList();
		
		try {
			NodeList list = null;
			if (parentElement instanceof IEditorInput) {
				list = fDoc.getChildNodes();
			} else if (parentElement instanceof Node) {
				Node node = (Node) parentElement;
				addTagCount(node);		
				list = node.getChildNodes();
		
				//Add the attributes of this node to the child list first
				NamedNodeMap map = node.getAttributes();
				if(map!=null) {
					int len = map.getLength();
					for (int i = 0; i < len; i++) {
						alist.add(map.item(i));
					}
				}
			}
			if(list!=null) {
				int len = list.getLength();
				//Add the children of the node to the child list now,but dont add comment or text node
				for (int i = 0; i < len; i++) {
					Node node = list.item(i);
					if (node.getNodeType() != Node.COMMENT_NODE){
						if(node.getNodeType() != Node.TEXT_NODE) {
							alist.add(node);
						}
					}
				}
			}
		} catch (RuntimeException e) {
			e.printStackTrace();
		}
		return alist.toArray();
	}
	/**
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(Object)
	 */
	public Object[] getElements(Object inputElement) {
		return getChildren(inputElement);
	}
	/**
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(Object)
	 */
	public Object getParent(Object element) {
		Object obj = null;
		if (element instanceof Node) {
			obj = ((Node) element).getParentNode();
		}
		return obj;
	}
	/**
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(Object)
	 */
	public boolean hasChildren(Object element) {
		boolean hasChild = false;
		
		if (element instanceof Node) {
			Node node = (Node) element;
			hasChild = ((Node) element).getChildNodes().getLength() > 0;
			
			//check if node has attributes, attributes are shown as children.
			//if node has attributes return true
			if(!hasChild){
				NamedNodeMap map = ((Node) element).getAttributes();
				if(map!=null)
					hasChild = map.getLength()>0;
			}
		}
		return hasChild;
	}
	/**
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(Viewer, Object, Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		if (newInput != null) {
			IDocument document = fProvider.getDocument(newInput);
			if (document != null) {
				try {
					fCountTags.clear();
					fDoc = DOMXHTMLParser.parse(document);
				} catch (RuntimeException e) {
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {
		fDoc = null;
		fProvider = null;
		fCountTags.clear();
		fCountTags = null;
	}
	
	public int getTagOccurenceCount(Node node){
		
		int size = fCountTags.size();
		int count = 1;
		for(int i=0;i<size;i++) {
			Node nodeTemp = (Node)fCountTags.get(i);
			if(nodeTemp!=node && nodeTemp.getNodeName().equals(node.getNodeName()))
				count++;
			if(nodeTemp==node)	
				break;
		}	
		return count;
	}
	
	private void addTagCount(Node node) {
		fCountTags.add(node);
	}
}