package org.zen.xhtml.editor.scanner;

import java.util.Locale;

import org.eclipse.jface.text.rules.ICharacterScanner;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.IWordDetector;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WordRule;

/**
 * Similar to Word Rule but case insensitive.
 * eg. for a given word say HTML the rule returns true
 * if it finds HTML,html,Html,HTml[as opposed to WordRule which returns true
 * if it finds only HTML]
 * 
 * @author Zenil
 * @version 1.0 
 */
public class CaseInsensitiveWordRule extends WordRule {

	private StringBuffer fBuffer = new StringBuffer();

	/**
	 * @see org.eclipse.jface.text.rules.WordRule#WordRule(IWordDetector, IToken)
	 */
	public CaseInsensitiveWordRule(IWordDetector detector, IToken defaultToken) {

		super(detector, defaultToken);
	}

	/**
	 * @see org.eclipse.jface.text.rules.WordRule#WordRule(IWordDetector, IToken)
	 */
	public CaseInsensitiveWordRule(IWordDetector detector) {

		super(detector);
	}

	/*
	 * @see IRule#evaluate
	 */
	public IToken evaluate(ICharacterScanner scanner) {
		int c = scanner.read();
		if (fDetector.isWordStart((char) c)) {
			if (fColumn == UNDEFINED || (fColumn == scanner.getColumn() - 1)) {

				fBuffer.setLength(0);
				do {
					fBuffer.append((char) c);
					c = scanner.read();
				} while (c != scanner.EOF && fDetector.isWordPart((char) c));
				scanner.unread();

				/* converting the scanned string to LOWER CASE(as HTML elements are 
				 * stored in this case in javax.swing.text.html.HTML class)
				 */
				String scannedStr = fBuffer.toString().toLowerCase(Locale.getDefault());
				IToken token = (IToken) fWords.get(scannedStr);
				if (token != null)
					return token;

				/*
				* Special handling for user defined attributes. Pre defined attributes
				* and user defined attributes are shown in the same color.
				* Check out if scanned word is a user defined attribute
				*/
				do {
					c = scanner.read();
					fBuffer.append((char) c);
				} while (Character.isWhitespace((char)c));
				
				/* if '=' is encountered then consider the word as an attribute */
				if(c =='='){
					//getting the token for predefined attribute 
					token = (IToken) fWords.get("size");
					if (token != null)
						return token;
				}

				if (fDefaultToken.isUndefined())
					unreadBuffer(scanner);

				return fDefaultToken;

			}
		}

		scanner.unread();
		return Token.UNDEFINED;
	}

	/**
	 * Returns the characters in the buffer to the scanner.
	 *
	 * @param scanner the scanner to be used
	 */
	protected void unreadBuffer(ICharacterScanner scanner) {
		for (int i = fBuffer.length() - 1; i >= 0; i--)
			scanner.unread();
	}
}
