package org.zen.xhtml.editor.scanner;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.Token;
import org.zen.xhtml.editor.IXHTMLEditorConstants;

/**
 * @author Zenil
 * @version 1.0 
 */
public class XHTMLPartitionScanner extends RuleBasedPartitionScanner {
	
	

	public XHTMLPartitionScanner() {

		List rules = new ArrayList();

		IToken htmlComment = new Token(IXHTMLEditorConstants.CONTENT_TYPE_HTML_COMMENT);
		IToken htmlTag = new Token(IXHTMLEditorConstants.CONTENT_TYPE_HTML_TAG);
		IToken htmlPI = new Token(IXHTMLEditorConstants.CONTENT_TYPE_HTML_PI);
		
		/*
		 * NOTE:Very IMP:
		 * The order of adding the rules is very important.
		 * The most restrictive rule should be added first.
		 * eg: Rule 3 should not be added first as it is the least restrictive.
		 * If it is added first none of the other rules will be detected!
		 * 
		 * There are alternate ways of acheiving the above fn. Check XMLEditor
		 * example for another implementation
		 */
		rules.add(new MultiLineRule("<!--", "-->", htmlComment));//$NON-NLS-1$ //Rule 1
		rules.add(new MultiLineRule("<?","?>",htmlPI));//$NON-NLS-1$ //Rule 2
		rules.add(new MultiLineRule("<",">",htmlTag));//$NON-NLS-1$  //Rule 3			

		IPredicateRule[] result= new IPredicateRule[rules.size()];
		rules.toArray(result);
		setPredicateRules(result);
	}
}

/*
 * alternateMethod()
 * 
 * No need to define htmlPI as a separate content type.
 * It can be default content type.For this,
 * htmlTag rule should be redefined to exclude '<?'  cases
 * 
 * Since "<?" sequence is now part of default content type ,
 * to detect it during presentation,a multiline rule can be added to 
 * the scanner associated with default content type
 * (which will be used in SourceViewerConfiguration presentation reconciler)
 * new multilinerule("<?","?>");
 * 
 * eg:
 * public class TagRule extends MultiLineRule {
		public TagRule(IToken token) {
			super("<", ">", token);
		}	
		protected boolean sequenceDetected(ICharacterScanner scanner, char[] sequence, boolean eofAllowed) {
			int c = scanner.read();
			if (sequence[0] == '<') {
			//returning false if '<?' sequence is detected.
				if (c == '?') {
					// processing instruction - abort
					scanner.unread();
					return false;
				}
			} else if (sequence[0] == '>') {
				scanner.unread();
			}
			return super.sequenceDetected(scanner, sequence, eofAllowed);
	}
}
 * 
 */