package org.zen.xhtml.editor.scanner;

import java.util.Vector;

import javax.swing.text.html.HTML;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WordRule;
import org.zen.xhtml.editor.ColorManager;
import org.zen.xhtml.editor.IXHTMLColorConstants;

/**
 * Scanner for content type HTML_TAG.i.e whenever a proper sequence of
 * '<....>' is encountered  this scanner is invoked.
 * Since <..."string"..> is a special sequence of the above, rules for
 * identifying these strings are added to this scanner.
 * Also rules for identifying PRE_DEFINED tags,USER_DEFINED tags
 * are done here
 * 
 * 
 * @author Zenil
 * @version 1.0 
 */
public class XHTMLTagScanner extends RuleBasedScanner {
	
	private ColorManager fColorManager;
	
	public XHTMLTagScanner(ColorManager colorManager) {
		fColorManager = colorManager;
		
		IToken default_tags = new Token(new TextAttribute(fColorManager.getColor(IXHTMLColorConstants.DEFAULT)));
		IToken user_defined_tags = new Token(new TextAttribute(fColorManager.getColor(IXHTMLColorConstants.USER_DEFINED_TAG)));
		IToken pre_defined_tags= new Token(new TextAttribute(fColorManager.getColor(IXHTMLColorConstants.PRE_DEFINED_TAG)));
		IToken attr_tags =  new Token(new TextAttribute(fColorManager.getColor(IXHTMLColorConstants.ATTRIBUTES)));
		
		//<,>,/(characters that are not covered by any of the rules) will be colored
		//by this default token.
		//Currently not required as default color is black .If you want to use other 
		//colors , u can set this.
		//setDefaultReturnToken(default_tags);
		
		/*
		 * NOTE: Adding rules properly might eke out better performance .
		 * Since in the XHTML document most likely tags will be encountered first
		 * and then strings, it is better to add word rule first
		 * and then string rules.
		 * This minimises the reparsing done by the scanner 
		 */
		Vector rules = new Vector();
		
		// Add word rule for pre-defined tags and attributes[pre-defined and user-defined]
		//if word is found ,but is not pre_defined tag or attribute ,consider it as 
		//user-defined tag
		WordRule wordRule= new CaseInsensitiveWordRule(new XHTMLWordDetector(),user_defined_tags);
		HTML.Tag tags[] = HTML.getAllTags();
		for(int i=0;i<tags.length;i++) {
			wordRule.addWord(tags[i].toString(),pre_defined_tags);
		}
		HTML.Attribute attributes[]=HTML.getAllAttributeKeys();
		for(int i=0;i<attributes.length;i++) {
			wordRule.addWord(attributes[i].toString(),attr_tags);
		}
		rules.add(wordRule);
		
		// Adding rules for single and double quotes
		IToken string_tags =	new Token(new TextAttribute(colorManager.getColor(IXHTMLColorConstants.STRING)));
		rules.add(new SingleLineRule("\"", "\"", string_tags, '\\'));
		rules.add(new SingleLineRule("'", "'", string_tags, '\\'));
		
		
		IRule[] result = new IRule[rules.size()];
		rules.copyInto(result);
		
		setRules(result);
		
	}

}
