package org.zen.xhtml.editor.util;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.runtime.Path;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;
import org.zen.xhtml.editor.XHTMLEditorPlugin;

/**
 * @author Zenil
 * @version 1.0 
 */
public class PluginImages {

	/**
	 * URL for the icon images
	 */
	private static URL fgIconBaseURL = null;

	/**
	 * Creates the URL for locating the icons.
	 */
	static {
		String pathSuffix = "icons/";//$NON-NLS-1$
		try {
			fgIconBaseURL = new URL(XHTMLEditorPlugin.getDefault().find(new Path("\\")), pathSuffix);//$NON-NLS-1$
		} catch (MalformedURLException e) {

		}
	}

	/**
	 * Image registry
	 */
	private final static ImageRegistry IMAGE_REGISTRY = new ImageRegistry();

	/**
	 * Image for Tag
	 */
	public static final String IMG_PRE_DEFINED_TAG = "Pre_Defined_Tag.gif";//$NON-NLS-1$
	
	/**
	 * Image for Tag
	 */
	public static final String IMG_USER_DEFINED_TAG = "User_Defined_Tag.gif";//$NON-NLS-1$


	/**
	 * Image for Attr
	 */
	public static final String IMG_ATTR = "Attribute.gif";//$NON-NLS-1$
	
	/**
	 * Image for Other
	 */
	public static final String IMG_OTHER = "Other.gif";//$NON-NLS-1$

	/**
	 * Image descriptor for tag
	 */
	public static final ImageDescriptor DESC_IMG_PRE_TAG = createManaged(IMG_PRE_DEFINED_TAG);

	/**
	 * Image descriptor for tag
	 */
	public static final ImageDescriptor DESC_IMG_USER_TAG = createManaged(IMG_USER_DEFINED_TAG);

	/**
	 * Image descriptor for attribute
	 */
	public static final ImageDescriptor DESC_IMG_ATTR = createManaged(IMG_ATTR);
	
	/**
	 * Image descriptor for other
	 */
	public static final ImageDescriptor DESC_IMG_OTHER = createManaged(IMG_OTHER);


	/**
	 * Returns the image managed under the given key in this registry.
	 *
	 * @param key the image's key
	 * @return the image managed under the given key
	 */
	public static Image get(String key) {

		Image image = IMAGE_REGISTRY.get(key);
		return image;
	}

	/**
	 * Manages the images using the image regisry
	 * @param name name of the icon
	 * @return the image descriptor
	 */
	private static ImageDescriptor createManaged(String name) {

		try {
			URL url = makeIconFileURL(name);
			ImageDescriptor result = ImageDescriptor.createFromURL(url);
			IMAGE_REGISTRY.put(name, result);
			return result;
		} catch (MalformedURLException e) {
			return ImageDescriptor.getMissingImageDescriptor();
		}
	}

	/**
	 * Makes the URL for the given icon file
	 * @param name name of the image
	 * @return the URL
	 */
	private static URL makeIconFileURL(String name) throws MalformedURLException {

		if (fgIconBaseURL == null) {
			throw new MalformedURLException();
		}
		return new URL(fgIconBaseURL, name);
	}
}
