package org.zen.xhtml.rulers;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.source.AnnotationRulerColumn;
import org.eclipse.jface.text.source.CompositeRuler;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.IVerticalRulerColumn;
import org.eclipse.jface.text.source.LineNumberRulerColumn;
import org.zen.xhtml.editor.ColorManager;
import org.zen.xhtml.editor.XHTMLEditorPlugin;
import org.zen.xhtml.editor.IXHTMLColorConstants;
import org.zen.xhtml.editor.preferencePage.IPreferenceConstants;

/**
 * @author Zenil
 * @version 1.0 
 */
public class VerticalRuler {

	private int fRulerWidth;
	private CompositeRuler fCompositeRuler;

	public VerticalRuler(int width) {
		fRulerWidth = width;
	}

	public IVerticalRuler createVerticalRuler() {
		fCompositeRuler = new CompositeRuler();
		fCompositeRuler.addDecorator(0, new AnnotationRulerColumn(fRulerWidth));
		if (isLineNumberRulerVisible())
			fCompositeRuler.addDecorator(1, createLineNumberRulerColumn());

		return fCompositeRuler;
	}
	/**
	 * Creates a new line number ruler column that is appropriately initialized.
	 */
	private IVerticalRulerColumn createLineNumberRulerColumn() {
		LineNumberRulerColumn lineNumberRulerColumn = new LineNumberRulerColumn();
		lineNumberRulerColumn.setForeground(ColorManager.getColorManager().getColor(IXHTMLColorConstants.BLACK));
		return lineNumberRulerColumn;
	}

	/**
	 * Return whether the line number ruler column should be 
	 * visible according to the preference store settings.
	 * @return <code>true</code> if the line numbers should be visible
	 */
	public boolean isLineNumberRulerVisible() {
		IPreferenceStore store = XHTMLEditorPlugin.getDefault().getPreferenceStore();
		return store.getBoolean(IPreferenceConstants.VRULER_PREFERENCE);
	}

	/**
	 * Shows the line number ruler column.
	 */
	public void showLineNumberRuler() {
		if (fCompositeRuler != null) {
			fCompositeRuler.addDecorator(1, createLineNumberRulerColumn());
		}
	}

	/**
	 * Hides the line number ruler column.
	 */
	public void hideLineNumberRuler() {
		if (fCompositeRuler != null) {
			fCompositeRuler.removeDecorator(1);
		}
	}

	public int getTotalRulerWidth() {
		int width=0;
		if(fCompositeRuler!=null) {
			width = fCompositeRuler.getWidth();
		}
		return width;
	}
}
