package org.zen.xhtml.views;

import java.text.MessageFormat;
import javax.swing.text.html.HTML;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;
import org.w3c.dom.Node;
import org.zen.xhtml.editor.XHTMLEditorPlugin;
import org.zen.xhtml.editor.parser.DOMXHTMLParser;
import org.zen.xhtml.editor.parser.XHTMLLabelProvider;
import org.zen.xhtml.editor.parser.XHTMLTreeContentProvider;
import org.zen.xhtml.editor.util.ToolTipHandler;

/**
 * @author Zenil
 * @version 1.0 
 */
public class XHTMLContentOutlinePage extends ContentOutlinePage {

	private TreeViewer fTreeViewer;
	private IEditorInput fInput;
	private IDocumentProvider fDocumentProvider;
	private ITextEditor fTextEditor;
	private XHTMLTreeContentProvider fTreeContentProvider;
	
	public XHTMLContentOutlinePage(IDocumentProvider provider, ITextEditor editor) {
		fDocumentProvider = provider;
		fTextEditor = editor;
		fTreeContentProvider = new XHTMLTreeContentProvider(fDocumentProvider);
		
	}
	/**
	 * @see org.eclipse.ui.views.contentoutline.ContentOutlinePage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		super.createControl(parent);
		fTreeViewer = getTreeViewer();
		fTreeViewer.setContentProvider(fTreeContentProvider);
		fTreeViewer.setLabelProvider(new XHTMLLabelProvider());
		fTreeViewer.addSelectionChangedListener(this);
		if (fInput != null)
			fTreeViewer.setInput(fInput);

		//this is necessary as only then the fCountTags for
		//all the tags  get updated	.If this is removed spl
		//handling to be done for getTagOccurenceCount()
		//as it can throw error.
		fTreeViewer.expandAll();

		ToolTipHandler tooltip = new ToolTipHandler(parent.getShell(), this);
		tooltip.activateHoverHelp(fTreeViewer.getTree());
	}

	/* (non-Javadoc)
	 * Method declared on ContentOutlinePage
	 */
	public void selectionChanged(SelectionChangedEvent event) {

		ISelection selection = event.getSelection();
		if (selection.isEmpty())
			fTextEditor.resetHighlightRange();
		else {
			Object sel = ((IStructuredSelection) selection).getFirstElement();
			if (sel instanceof Node && sel != null) {

				String name = ((Node) sel).getNodeName();
				int occurence = fTreeContentProvider.getTagOccurenceCount((Node) sel);
				int offset = 0 - name.length();
				IDocument doc = fDocumentProvider.getDocument(fInput);
				for (int count = 0; count < occurence; count++) {
					try {
						offset = offset + name.length();
						offset = doc.search(offset, name, true, true, true);
						//if previous character is '/',it is end tag, ignore it
						//and continue searching
						if (doc.getChar(offset - 1) == '/') {
							count--;
						}

					} catch (BadLocationException e) {
					}
				}
				fTextEditor.selectAndReveal(offset, name.length());
			}
		}
	}

	/**
	 * Sets the input of the outline page
	 */
	public void setInput(IEditorInput input) {
		fInput = input;
		update();
	}

	/**
	 * Updates the outline page.
	 */
	public void update() {
		TreeViewer viewer = getTreeViewer();
		if (viewer != null) {
			Control control = viewer.getControl();
			if (control != null && !control.isDisposed()) {
				control.setRedraw(false);
				viewer.setInput(fInput);
				viewer.expandAll();
				control.setRedraw(true);
			}
		}
	}

	public String getToolTipText(Object object) {
		String tooltip = null;
		if (object instanceof Node) {
			Node node = (Node) object;
			String name = node.getNodeName();
			if (node.getNodeType() == Node.ATTRIBUTE_NODE) {
				if (HTML.getAttributeKey(name) != null) {
					tooltip = MessageFormat.format(XHTMLEditorPlugin.getResourceString("XHTMLContentOutlinePage.pre_defined_attr"), new String[] { name }); //$NON-NLS-1$
				} else {
					tooltip = MessageFormat.format(XHTMLEditorPlugin.getResourceString("XHTMLContentOutlinePage.user_defined_attr"), new String[] { name }); //$NON-NLS-1$
				}
			} else if (node.getNodeType() == Node.ELEMENT_NODE) {
				if (HTML.getTag(name) != null) {
					tooltip = MessageFormat.format(XHTMLEditorPlugin.getResourceString("XHTMLContentOutlinePage.pre_defined_tag"), new String[] { name }); //$NON-NLS-1$
				} else {
					tooltip = MessageFormat.format(XHTMLEditorPlugin.getResourceString("XHTMLContentOutlinePage.user_defined_tag"), new String[] { name }); //$NON-NLS-1$
				}
			}
		}
		return tooltip;
	}

}
